'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Log = exports.defaultConfig = exports.indent = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

exports.formatError = formatError;
exports.configFromEnvironment = configFromEnvironment;
exports.fromEnvironment = fromEnvironment;
exports.fromEnvironmentVariables = fromEnvironmentVariables;

var _chalk = require('chalk');

var _dateformat = require('dateformat');

var _dateformat2 = _interopRequireDefault(_dateformat);

var _jestMessageUtil = require('jest-message-util');

var _path = require('path');

var _purdy = require('purdy');

var _stripAnsi = require('strip-ansi');

var _stripAnsi2 = _interopRequireDefault(_stripAnsi);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var indent = exports.indent = function indent(string, length) {
  var char = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : ' ';
  return '' + string.replace(/\n/g, '\n' + char.repeat(length));
};

function formatError(error) {
  var _separateMessageFromS = (0, _jestMessageUtil.separateMessageFromStack)(error.stack),
      message = _separateMessageFromS.message,
      stack = _separateMessageFromS.stack;

  var stackTrace = (0, _jestMessageUtil.formatStackTrace)(stack, { rootDir: (0, _path.dirname)((0, _path.dirname)(__dirname)), testMatch: [] }, { noStackTrace: false });
  return (0, _chalk.red)(message) + '\n' + stackTrace.replace(/\n {4}/g, '\n') + '\n';
}

function formatObject(object) {
  var str = (0, _purdy.stringify)(object, { indent: 2, arrayIndex: false });
  return (0, _chalk.grey)((0, _stripAnsi2.default)(str).length < 200 ? str.replace(/\n */g, ' ') : '\n' + str + '\n');
}

function format(value) {
  if (value instanceof Error) {
    return formatError(value);
  }
  return typeof value === 'string' ? value : formatObject(value);
}

var defaultConfig = exports.defaultConfig = {
  levels: ['emergency', 'alert', 'critical', 'error', 'warning', 'notice', 'info', 'debug'],
  colors: [_chalk.red.bold, _chalk.red, _chalk.red.bold, _chalk.red, _chalk.magenta, _chalk.yellow, _chalk.blue, _chalk.grey],
  inverse: [true, true],
  labels: ['EMERG', 'ALERT', 'CRITI', 'ERROR', ' WARN', ' NOTE', ' INFO', 'DEBUG'],
  separator: '•',
  stream: function stream(level) {
    return level <= 4 ? process.stderr : process.stdout;
  },
  format: format,
  displayTime: true,
  displayTimeFormat: 'yyyy-mm-dd HH:MM:ss.l',
  level: 'info'
};

var Log = exports.Log = function () {
  function Log() {
    var _this = this;

    var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _classCallCheck(this, Log);

    // Options
    Object.assign(this, defaultConfig, options);
    this.levelValue = this.levels.indexOf(this.level);

    // Level methods
    this.levels.forEach(function (name, logLevel) {
      _this[name] = function () {
        for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
          args[_key] = arguments[_key];
        }

        return _this.print.apply(_this, [logLevel].concat(args));
      };
    });
  }

  _createClass(Log, [{
    key: 'print',
    value: function print(level) {
      if (level <= this.levelValue) {
        var time = this.displayTime ? (0, _dateformat2.default)(Date.now(), this.displayTimeFormat) : '';
        var color = this.colors[level];
        var labelColor = this.inverse[level] ? color.inverse : color;
        var label = '' + this.labels[level];

        for (var _len2 = arguments.length, args = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
          args[_key2 - 1] = arguments[_key2];
        }

        var messages = args.map(this.format);
        var message = messages.join(messages.some(function (s) {
          return s.includes('\n');
        }) ? '\n' : ' ');
        var stream = typeof this.stream === 'function' ? this.stream(level) : this.stream;
        var separator = this.separator;

        var header = (time && (0, _chalk.grey)(time + ' ' + separator)) + ' ' + labelColor(label) + ' ' + color(separator) + ' ';
        stream.write('' + header + indent(message, (0, _stripAnsi2.default)(header).length) + '\n');
      }
    }
  }]);

  return Log;
}();

function configFromEnvironment() {
  var environment = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'development';
  var aliases = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : { testing: 'test', staging: 'production' };
  var defaults = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {
    development: { level: 'info', displayTime: false },
    production: { level: 'info' },
    test: { level: 'critical', displayTime: false }
  };

  var alias = aliases[environment];
  if (alias) {
    return configFromEnvironment(alias, defaults, aliases);
  }
  return defaults[environment];
}

function fromEnvironment() {
  return new Log(configFromEnvironment.apply(undefined, arguments));
}

function fromEnvironmentVariables() {
  var environment = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : process.env.NODE_ENV;
  var level = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : process.env.LOG || process.env.LOG_LEVEL;
  var displayTime = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : process.env.LOG_TIME;

  return new Log(Object.assign({}, configFromEnvironment(environment), level ? { level: level } : {}, { displayTime: ['1', 'true'].indexOf(displayTime) !== -1 }));
}